@@ -142,7 +142,7 @@ static const byte lop2hop[12][9] = {
 	/* X86_OPTYPE_IMM */
 	{TYPE_I, TYPE_Is, TYPE_J, TYPE_A, TYPE_Ix, TYPE_I4},
 	/* X86_OPTYPE_REG */
-	{TYPE_R, TYPE_Rx, TYPE_RXx, TYPE_G, TYPE_E, TYPE_MR},
+	{TYPE_R, TYPE_Rx, TYPE_RXx, TYPE_G, TYPE_E, TYPE_MR, TYPE_RV},
 	/* X86_OPTYPE_SEG */
 	{TYPE_S, TYPE_Sx},
 	/* X86_OPTYPE_MEM */
@@ -399,20 +399,20 @@ asm_code *x86asm::encode(asm_insn *asm_insn, int options, CPU_ADDR cur_address)
 		set_error_msg(X86ASM_ERRMSG_INVALID_PREFIX);
 	} else {
 		match_fopcodes(insn);
-        	match_opcodes(x86_insns_ext, insn, X86ASM_PREFIX_0F, MATCHOPNAME_MATCH_IF_NOOPPREFIX);
-        	match_opcodes(x86_insns_ext_66, insn, X86ASM_PREFIX_0F, MATCHOPNAME_MATCH_IF_OPPREFIX);
-        	match_opcodes(x86_insns_ext_f2, insn, X86ASM_PREFIX_F20F, MATCHOPNAME_MATCH_IF_NOOPPREFIX);
-        	match_opcodes(x86_insns_ext_f3, insn, X86ASM_PREFIX_F30F, MATCHOPNAME_MATCH_IF_NOOPPREFIX);
-        	match_opcodes(x86_opc_group_insns[0], insn, X86ASM_PREFIX_0F38, MATCHOPNAME_MATCH_IF_NOOPPREFIX);
-        	match_opcodes(x86_opc_group_insns[1], insn, X86ASM_PREFIX_660F38, MATCHOPNAME_MATCH_IF_OPPREFIX);
-        	match_opcodes(x86_opc_group_insns[2], insn, X86ASM_PREFIX_F20F38, MATCHOPNAME_MATCH_IF_NOOPPREFIX);
-        	match_opcodes(x86_opc_group_insns[3], insn, X86ASM_PREFIX_0F3A, MATCHOPNAME_MATCH_IF_NOOPPREFIX);
-        	match_opcodes(x86_opc_group_insns[4], insn, X86ASM_PREFIX_660F3A, MATCHOPNAME_MATCH_IF_OPPREFIX);
-        	match_opcodes(x86_opc_group_insns[5], insn, X86ASM_PREFIX_0F7A, MATCHOPNAME_MATCH_IF_NOOPPREFIX);
-        	match_opcodes(x86_opc_group_insns[6], insn, X86ASM_PREFIX_0F7B, MATCHOPNAME_MATCH_IF_NOOPPREFIX);
-        	match_opcodes(x86_opc_group_insns[7], insn, X86ASM_PREFIX_0F24, MATCHOPNAME_MATCH_IF_NOOPPREFIX);
-        	match_opcodes(x86_opc_group_insns[8], insn, X86ASM_PREFIX_0F25, MATCHOPNAME_MATCH_IF_NOOPPREFIX);
-        	match_vex_opcodes(insn);
+		match_opcodes(x86_insns_ext, insn, X86ASM_PREFIX_0F, MATCHOPNAME_MATCH_IF_NOOPPREFIX);
+		match_opcodes(x86_insns_ext_66, insn, X86ASM_PREFIX_0F, MATCHOPNAME_MATCH_IF_OPPREFIX);
+		match_opcodes(x86_insns_ext_f2, insn, X86ASM_PREFIX_F20F, MATCHOPNAME_MATCH_IF_NOOPPREFIX);
+		match_opcodes(x86_insns_ext_f3, insn, X86ASM_PREFIX_F30F, MATCHOPNAME_MATCH_IF_NOOPPREFIX);
+		match_opcodes(x86_opc_group_insns[0], insn, X86ASM_PREFIX_0F38, MATCHOPNAME_MATCH_IF_NOOPPREFIX);
+		match_opcodes(x86_opc_group_insns[1], insn, X86ASM_PREFIX_660F38, MATCHOPNAME_MATCH_IF_OPPREFIX);
+		match_opcodes(x86_opc_group_insns[2], insn, X86ASM_PREFIX_F20F38, MATCHOPNAME_MATCH_IF_NOOPPREFIX);
+		match_opcodes(x86_opc_group_insns[3], insn, X86ASM_PREFIX_0F3A, MATCHOPNAME_MATCH_IF_NOOPPREFIX);
+		match_opcodes(x86_opc_group_insns[4], insn, X86ASM_PREFIX_660F3A, MATCHOPNAME_MATCH_IF_OPPREFIX);
+		match_opcodes(x86_opc_group_insns[5], insn, X86ASM_PREFIX_0F7A, MATCHOPNAME_MATCH_IF_NOOPPREFIX);
+		match_opcodes(x86_opc_group_insns[6], insn, X86ASM_PREFIX_0F7B, MATCHOPNAME_MATCH_IF_NOOPPREFIX);
+		match_opcodes(x86_opc_group_insns[7], insn, X86ASM_PREFIX_0F24, MATCHOPNAME_MATCH_IF_NOOPPREFIX);
+		match_opcodes(x86_opc_group_insns[8], insn, X86ASM_PREFIX_0F25, MATCHOPNAME_MATCH_IF_NOOPPREFIX);
+		match_vex_opcodes(insn);
 	}
 	if (error) {
 		free_asm_codes();
@@ -688,6 +688,15 @@ bool x86asm::encode_vex_insn(x86asm_insn *insn, x86opc_vex_insn *opcode, int opc
 		clearcode();
 		return false;
 	}
+
+	if (addrsize == X86_ADDRSIZE64) {
+		if (eaddrsize == X86_ADDRSIZE16) return false;
+		if (eaddrsize == X86_ADDRSIZE32) emitbyte(0x67);
+	} else {
+		if (eaddrsize != addrsize) emitbyte(0x67);
+	}
+
+
 	switch (insn->segprefix) {
 	case X86_PREFIX_ES: emitbyte(0x26); break;
 	case X86_PREFIX_CS: emitbyte(0x2e); break;
@@ -704,8 +713,8 @@ bool x86asm::encode_vex_insn(x86asm_insn *insn, x86opc_vex_insn *opcode, int opc
 		}
 	}
 
-	if ((opcode->vex & 0x30) == 0x10 // 0x0f-opcode
-	 && !(rexprefix & 3)          // no rexb/rexx
+	if ((opcode->vex & 0x3c) == 0x4 // 0x0f-opcode
+	 && !(rexprefix & 3)            // no rexb/rexx
 	 && !(opcode->vex & W1)) {
 		// use short vex prefix
 		emitbyte(0xc5);
@@ -715,9 +724,13 @@ bool x86asm::encode_vex_insn(x86asm_insn *insn, x86opc_vex_insn *opcode, int opc
 			| (opcode->vex & 0x3);
 		emitbyte(vex);
 	} else {
-		// use long vex prefix
-		emitbyte(0xc4);
-		byte vex = (~rexprefix & 7) << 5 | ((opcode->vex & 0x30) >> 4);
+		// use long vex/xop prefix
+		if (opcode->vex & _0f24) {
+			emitbyte(0x8f);
+		} else {
+			emitbyte(0xc4);
+		}
+		byte vex = (~rexprefix & 7) << 5 | ((opcode->vex & 0x3c) >> 2);
 		emitbyte(vex);
 		vex = (opcode->vex & W1) | ((~vexvvvv & 0xf) << 3) 
 			| (opcode->vex & _256) >> 4 
@@ -821,11 +834,13 @@ bool x86asm::encode_modrm(x86_insn_op *op, char size, bool allow_reg, bool allow
 		if (!allow_reg) return false;
 		emitmodrm_mod(3);
 		emitmodrm_rm(op->xmm);
+		if (op->xmm > 7) rexprefix |= rexb;
 		break;
 	case X86_OPTYPE_YMM:
 		if (!allow_reg) return false;
 		emitmodrm_mod(3);
 		emitmodrm_rm(op->ymm);
+		if (op->ymm > 7) rexprefix |= rexb;
 		break;
 	default:
 		return false;
@@ -838,9 +853,10 @@ bool x86asm::encode_modrm_v(const x86addrcoding (*modrmc)[3][8], x86_insn_op *op
 	if (op->mem.scale > 1) return false;
 	for (int mod=0; mod < 3; mod++) {
 		for (int rm=0; rm < 8; rm++) {
-			const x86addrcoding *c=&(*modrmc)[mod][rm];
-			int r1=c->reg1, r2=c->reg2;
-			if (r2 == (op->mem.base&~8)) {
+			const x86addrcoding *c = &(*modrmc)[mod][rm];
+			int r1 = c->reg1;
+			int r2 = c->reg2;
+			if (r2 == (op->mem.base & ~8)) {
 				int t = r1;
 				r1 = r2;
 				r2 = t;
@@ -986,6 +1002,7 @@ bool x86asm::encode_op(x86_insn_op *op, x86opc_insn_op *xop, int *esize, int eop
 		break;
 	case TYPE_R:
 		/* rm of ModR/M picks general register */
+		emitmodrm_mod(3);
 		emitmodrm_rm(op->reg);
 		// fall throu
 	case TYPE_Rx:
@@ -994,6 +1011,10 @@ bool x86asm::encode_op(x86_insn_op *op, x86opc_insn_op *xop, int *esize, int eop
 	case TYPE_RXx:
 		/* extra picks register, no REX */
 		return true;
+	case TYPE_RV:
+		/* VEX.vvvv picks general register */
+		vexvvvv = op->reg;
+		return true;
 	case TYPE_S:
 		/* reg of ModR/M picks segment register */
 		emitmodrm_reg(op->seg);
@@ -1040,7 +1061,7 @@ bool x86asm::encode_op(x86_insn_op *op, x86opc_insn_op *xop, int *esize, int eop
 			if (drexoc0 == 0) {
 				emitmodrm_mod(3);
 				emitmodrm_rm(op->xmm);
-				if (op->xmm > 7) rexprefix |= rexb;				
+				if (op->xmm > 7) rexprefix |= rexb;
 			} else {
 				emitmodrm_reg(op->xmm);
 				if (op->xmm > 7) rexprefix |= rexr;
@@ -1057,7 +1078,8 @@ bool x86asm::encode_op(x86_insn_op *op, x86opc_insn_op *xop, int *esize, int eop
 		}
 		break;
 	case TYPE_W:
-		/* ModR/M (XMM reg or memory) */
+	case TYPE_X:
+		/* ModR/M (XMM/YMM reg or memory) */
 		if (!encode_modrm(op, xop->size, true, true, eopsize, eaddrsize)) return false; //XXX
 		psize = esizeop(xop->size, eopsize); //XXX
 		break;
@@ -1084,11 +1106,6 @@ bool x86asm::encode_op(x86_insn_op *op, x86opc_insn_op *xop, int *esize, int eop
 		emitmodrm_rm(op->ymm);
 		if (op->ymm > 7) rexprefix |= rexb;
 		break;
-	case TYPE_X:
-		/* ModR/M (YMM reg or memory) */
-		if (!encode_modrm(op, xop->size, true, true, eopsize, eaddrsize)) return false; //XXX
-		psize = esizeop(xop->size, eopsize); //XXX
-		break;
 	}
 	if (!psize) {
 //		set_error_msg(X86ASM_ERRMSG_INTERNAL"FIXME: size ??? %s, %d\n", __FILE__, __LINE__);
@@ -1128,25 +1145,16 @@ bool x86asm::encode_sib_v(x86_insn_op *op, int mindispsize, int *_ss, int *_inde
 	}
 	if (index != X86_REG_NO) {
 		switch (scale) {
-		case 1:
-			ss = 0;
-			break;
-		case 2:
-			ss = 1;
-			break;
-		case 4:
-			ss = 2;
-			break;
-		case 8:
-			ss = 3;
-			break;
-		default:
-			return 0;
-		}				
+		case 1: ss = 0; break;
+		case 2: ss = 1; break;
+		case 4: ss = 2; break;
+		case 8: ss = 3; break;
+		default: return 0;
+		}
 	} else {
 		ss = 0;
 		index = 4;
-	}		
+	}
 	switch (mindispsize) {
 	case 0:
 		mod = 0;
@@ -1687,7 +1695,7 @@ void x86asm::match_fopcodes(x86asm_insn *insn)
 				int eaddrsize = addrsize;
 				for (int k=0; k < 2; k++) {
 					if (match_allops(insn, x86_modfloat_group_insns[i][j].op, 4, opsize, eaddrsize)) {
-						if (encode_insn(insn, &x86_modfloat_group_insns[i][j], j<<3, -1, X86ASM_PREFIX_D8+i, opsize, eaddrsize)) {
+						if (encode_insn(insn, &x86_modfloat_group_insns[i][j], -1, j, X86ASM_PREFIX_D8+i, opsize, eaddrsize)) {
 							pushcode();
 							newcode();
 						}
@@ -1709,7 +1717,7 @@ void x86asm::match_fopcodes(x86asm_insn *insn)
 				namefound |= n;
 				if (n != MATCHOPNAME_NOMATCH) {
 					if (match_allops(insn, x86_float_group_insns[i][j].insn.op, 4, opsize, addrsize)) {
-						if (encode_insn(insn, &x86_float_group_insns[i][j].insn, 0xc0 | j<<3, -1, X86ASM_PREFIX_D8+i, opsize, addrsize)) {
+						if (encode_insn(insn, &x86_float_group_insns[i][j].insn, -1, 0x800 | j, X86ASM_PREFIX_D8+i, opsize, addrsize)) {
 							pushcode();
 							newcode();
 						}
@@ -1725,7 +1733,7 @@ void x86asm::match_fopcodes(x86asm_insn *insn)
 						int eaddrsize = addrsize;
 						for (int l=0; l < 2; l++) {
 							if (match_allops(insn, group[k].op, 4, opsize, eaddrsize)) {
-								if (encode_insn(insn, &group[k], 0xc0 | j<<3 | k, -1, X86ASM_PREFIX_D8+i, opsize, eaddrsize)) {
+								if (encode_insn(insn, &group[k], -1, 0x800 | k<<3 | j, X86ASM_PREFIX_D8+i, opsize, eaddrsize)) {
 									pushcode();
 									newcode();
 								}
@@ -1879,13 +1887,13 @@ bool x86asm::opmem(x86asm_insn *asm_insn, x86_insn_op *op, const char *s)
 
 	tok(&s, token, sizeof token, sep);
 	
-	static const char *types[] = {"byte", "word", "dword", "pword", "qword", "oword", "ymmword", "single", "double", "extended"};
-	static byte type_size[] = {1, 2, 4, 6, 8, 16, 32, 4, 8, 10};
+	static const char *types[] = {"byte", "word", "dword", "pword", "qword", "oword", "xmmword", "ymmword", "single", "double", "extended"};
+	static byte type_size[] = {1, 2, 4, 6, 8, 16, 16, 32, 4, 8, 10};
 	// typecast
 	for (uint i=0; i < sizeof types / sizeof types[0]; i++) {
 		if (strcmp(token, types[i]) == 0) {
 			hsize = type_size[i];
-			if (i >= 6) floatptr = true;
+			if (i >= 8) floatptr = true;
 			break;
 		}
 	}
@@ -2103,7 +2111,7 @@ bool x86asm::translate_str(asm_insn *asm_insn, const char *s)
 
 	/**/
 	splitstr(s, insn->n, sizeof insn->n, (char**)&opp, 256);
-	insn->name=insn->n;
+	insn->name = insn->n;
 	for (int i=0; i<5; i++) {
 		if (!*op[i]) break;
 
