@@ -34,8 +34,8 @@ extern format_viewer_if htclassimage_if;
 class ht_class_aviewer: public ht_aviewer {
 public:
 	ht_class_shared_data *class_shared;
-	ht_streamfile *file;
-		   void init(bounds *b, char *desc, int caps, ht_streamfile *file, ht_format_group *format_group, Analyser *Analy, ht_class_shared_data *class_shared);
+	File *file;
+		void init(Bounds *b, const char *desc, int caps, File *file, ht_format_group *format_group, Analyser *Analy, ht_class_shared_data *class_shared);
 	virtual void setAnalyser(Analyser *a);
 };
 
