@@ -102,26 +102,24 @@ void ht_pe::init(Bounds *b, File *file, format_viewer_if **ifs, ht_format_group
 	pe_shared->opt_magic = createHostInt(&pe_shared->opt_magic, sizeof pe_shared->opt_magic, little_endian);
 	file->seek(header_ofs+4+sizeof pe_shared->coffheader);
 	switch (pe_shared->opt_magic) {
-		case COFF_OPTMAGIC_PE32: {
-			file->readx(&pe_shared->pe32.header, sizeof pe_shared->pe32.header);
-			createHostStruct(&pe_shared->pe32.header, COFF_OPTIONAL_HEADER32_struct, little_endian);
-			file->readx(&pe_shared->pe32.header_nt, sizeof pe_shared->pe32.header_nt);
-			createHostStruct(&pe_shared->pe32.header_nt, PE_OPTIONAL_HEADER32_NT_struct, little_endian);
-			for (uint i=0; i<PE_NUMBEROF_DIRECTORY_ENTRIES; i++) {
-				createHostStruct(&pe_shared->pe32.header_nt.directory[i], PE_DATA_DIRECTORY_struct, little_endian);
-			}
-			break;
+	case COFF_OPTMAGIC_PE32:
+		file->readx(&pe_shared->pe32.header, sizeof pe_shared->pe32.header);
+		createHostStruct(&pe_shared->pe32.header, COFF_OPTIONAL_HEADER32_struct, little_endian);
+		file->readx(&pe_shared->pe32.header_nt, sizeof pe_shared->pe32.header_nt);
+		createHostStruct(&pe_shared->pe32.header_nt, PE_OPTIONAL_HEADER32_NT_struct, little_endian);
+		for (uint i=0; i<PE_NUMBEROF_DIRECTORY_ENTRIES; i++) {
+			createHostStruct(&pe_shared->pe32.header_nt.directory[i], PE_DATA_DIRECTORY_struct, little_endian);
 		}
-		case COFF_OPTMAGIC_PE64: {
-			file->readx(&pe_shared->pe64.header, sizeof pe_shared->pe64.header);
-			createHostStruct(&pe_shared->pe64.header, COFF_OPTIONAL_HEADER64_struct, little_endian);
-			file->readx(&pe_shared->pe64.header_nt, sizeof pe_shared->pe64.header_nt);
-			createHostStruct(&pe_shared->pe64.header_nt, PE_OPTIONAL_HEADER64_NT_struct, little_endian);
-			for (uint i=0; i<PE_NUMBEROF_DIRECTORY_ENTRIES; i++) {
-				createHostStruct(&pe_shared->pe64.header_nt.directory[i], PE_DATA_DIRECTORY_struct, little_endian);
-			}
-			break;
+		break;
+	case COFF_OPTMAGIC_PE64:
+		file->readx(&pe_shared->pe64.header, sizeof pe_shared->pe64.header);
+		createHostStruct(&pe_shared->pe64.header, COFF_OPTIONAL_HEADER64_struct, little_endian);
+		file->readx(&pe_shared->pe64.header_nt, sizeof pe_shared->pe64.header_nt);
+		createHostStruct(&pe_shared->pe64.header_nt, PE_OPTIONAL_HEADER64_NT_struct, little_endian);
+		for (uint i=0; i<PE_NUMBEROF_DIRECTORY_ENTRIES; i++) {
+			createHostStruct(&pe_shared->pe64.header_nt.directory[i], PE_DATA_DIRECTORY_struct, little_endian);
 		}
+		break;
 	}
 
 	/* read section headers */
